/*******************************************************************************
	Bjrn Clausen
	Materials Research Department
	Ris National Laboratory
	P.O. Box 49
	4000  Roskilde
	Denmark
	Ph.: +45 4677 4677

*******************************************************************************/
#define NR_END 1
#include<string.h>
#include<math.h>
#include<stdlib.h>
#include<stdio.h>
FILE *out,*ini;

char filename[81],string[81],ch;
int i,j,max_grain,hlaw,mtr,n_cyc,max_ref,max_type;
double ep_d,max_strain,ny,tau0,q_ratio,h_ratio,ins_ang,h_final,h_exp,*max_cyc;

void nrerror(char error_text[])
/* Standard error handler */
{
	fprintf(stderr,"Run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	ch=getc(stdin);
	exit(1);
}

double *vector(int nl, int nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl+NR_END;
}

int main(void)
{


/* Selecting the file containing the initial orientation of the grains*/
	printf("\nType the name of the file that contains the Euler angels for the grains\n");
	printf("(default: \"random.dat\"): ");
	gets(filename);
	if(strlen(filename)<1)
		strcpy(filename,"random.dat");


/* Definition of the number of grains*/
	while(1)
	{	printf("\nType the number of grains (default 200): ");
		gets(string);
		max_grain=atoi(string);
		if(strlen(string)<1)
		{
			max_grain = 200;
			break;
		}
		if(max_grain<=0 || max_grain>10000)
		{
			printf("Number of grains must be greater than zero and max 10000!\n");
			continue;
		}
		else
			break;
	}




/* Definition of the strain increment and the number of iterations*/
	while(1)
	{
		printf("\nType the strain increment in %% (default = 0.01): ");
		gets(string);
		ep_d=atof(string)/100;
		if(strlen(string)<1)
		{
			ep_d=0.0001;
			break;
		}
		if(ep_d>0)
			break;
		else
		{
			printf("The strain increment must be greater than zero!\n");
			continue;
		}
	}





	while(1)
	{
		printf("\nSelect the controling parameter for the maximum load");
					 printf("\nType 1 for strain, 2 for stress or 3 for plastic strain");
					 printf("\n(default is 1): ");
		gets(string);
		max_type = *string;
		if(max_type=='1' || strlen(string)<1)
		{
			max_type='1';
			printf("You chose strain");
			break;
		}
		else if(max_type=='2')
		{
			printf("You chose stress");
			break;
		}
		else if(max_type=='3')
		{
			printf("You chose plastic strain");
			break;
		}
		else
		{
			printf("\nType 1, 2 or 3 !!!");
			continue;
		}
	}

if(max_type=='1')
{
	while(1)
	{
		printf("\nType the maximum strain in %% (default = 3%%): ");
		gets(string);
		max_strain = atof(string)/100;
		if(strlen(string)<1)
		{
			max_strain=0.03;
			break;
		}
		else if(max_strain>0 && max_strain<=0.1)
			break;
		else
		{
			printf("Max 10%% strain!");
			continue;
		}
	}
}
else if(max_type=='2')
{
	while(1)
	{
		printf("\nType the maximum stress in MPa (default = 100MPa): ");
		gets(string);
		max_strain = atof(string);
		if(strlen(string)<1)
		{
			max_strain=100;
			break;
		}
		else if(max_strain>0 && max_strain<=500)
			break;
		else
		{
			printf("Max 500 MPa !");
			continue;
		}
	}
}
else
{
	while(1)
	{
		printf("\nType the maximum plastic strain in %% \n(default = 3%%): ");
		gets(string);
		max_strain = atof(string)/100;
		if(strlen(string)<1)
		{
			max_strain=0.03;
			break;
		}
		else if(max_strain>0 && max_strain<=0.1)
			break;
		else
		{
			printf("Max 10%% strain!");
			continue;
		}
	}
}



/*Definition of material*/
	printf("\nSelection of material.\nType 1 for ISO (isotropic), 2 for aluminium, 3 for copper,");
	printf("\n4 for hybrid or 5 for austenittic stainless steel");
	while(1)
	{
		printf("\nDefault is aluminium (2): ");
		gets(string);
		mtr = *string;
		if(mtr=='1')
		{
			printf("You chose: ISO\n");
			break;
		}
		else if(strlen(string)<1 || mtr == '2')
		{
			mtr = '2';
			printf("You chose: aluminium\n");
			break;
		}
		else if(mtr == '3')
		{
			printf("You chose: copper\n");
			break;
		}
		else if(mtr == '4')
		{
			printf("You chose: hybrid\n");
			break;
		}
		else if(mtr == '5')
		{
			printf("You chose: austenitic stainless steel\n");
			break;
		}
		else
		{
			printf("Type 1, 2, 3, 4 or 5! ");
			continue;
		}
	}



/* Definition of tau0*/
	while(1)
	{
		if(mtr=='1')
			printf("\nType tau0 for the chosen material (default = 65 MPa): ");
		else if(mtr=='2' || mtr=='4')
			printf("\nType tau0 for the chosen material (default = 10.9 MPa): ");
		else if(mtr=='3')
			printf("\nType tau0 for the chosen material (default = 15.0 MPa): ");
		else
			printf("\nType tau0 for the chosen material (default = 87.0 MPa): ");
		gets(string);
		tau0 = atof(string);
		if(strlen(string)<1)
		{
			if(mtr=='1')
				tau0 = 65;
			else if(mtr=='2' || mtr=='4')
				tau0 = 10.9;
			else if(mtr=='3')
				tau0 = 15.0;
			else
				tau0 = 87.0;
			break;
		}
		else if(tau0<=0)
		{
			printf("Tau0 must be greater than zero\n");
			continue;
		}
		else
			break;
	}



/* Hardening!*/
	printf("Hill hardening is used: h_ij = h_gamma(q + (1-q)*delta_ij.\n");
	printf("\nType the ratio between latent hardening and primary hardening q.");
	while(1)
	{
		printf("\n(0<=q<=1.4 default 1.01): ");
		gets(string);
		q_ratio = atof(string);
		if(strlen(string)<1)
		q_ratio = 1.01;
		if(q_ratio < 0 || q_ratio > 1.4)
		{
			printf("\n 0 <= q <= 1.4 !!\n");
			continue;
		}
		else
			break;
	}/* while(1)*/


	printf("\n\nExponential hardening:\n");
	printf("h_gamma = h_final(1-(h_ratio-1)*(1-exp(-h_exp*gamma_tot))\n");
	while(1)
	{
		if(mtr=='1')
			printf("\nType the final hardening coefficient (default is h_final = 140): ");
		else if(mtr=='2' || mtr=='4')
			printf("\nType the final hardening coefficient (default is h_final = 40): ");
		else if(mtr=='3')
			printf("\nType the final hardening coefficient (default is h_final = 250): ");
		else
			printf("\nType the final hardening coefficient (default is h_final = 500): ");
		gets(string);
		h_final=atof(string);
		if(strlen(string)<1)
		{
			if(mtr=='1')
				h_final=140;
			else if(mtr=='2' || mtr=='4')
				h_final=40;
			else if(mtr=='3')
				h_final=250;
			else
				h_final=500;
			break;
		}
		else if(h_final>=0)
			break;
		else
		{
			printf("h_final must be non-negative!\n");
			continue;
		}
	}


	while(1)
	{
		if(mtr=='1')
			printf("\nType the value for the hardening ratio h_ratio (default 20): ");
		else if(mtr=='2' || mtr=='4')
			printf("\nType the value for the hardening ratio h_ratio (default 5): ");
		else if(mtr=='3')
			printf("\nType the value for the hardening ratio h_ratio (default 20): ");
		else
			printf("\nType the value for the hardening ratio h_ratio (default 5): ");
		gets(string);
		h_ratio = atof(string);
		if(strlen(string)<1)
		{
			if(mtr=='1')
				h_ratio = 20;
			else if(mtr=='2' || mtr=='4')
				h_ratio = 5;
			else if(mtr=='3')
				h_ratio = 20;
			else
				h_ratio = 5;
		}
		if(h_ratio < 0 || h_ratio > 5000)
		{
			printf("\n 0 <= h_ratio <= 5000 !!\n");
			continue;
		}
		else
			break;
	}

	while(1)
	{
		if(mtr=='1')
			printf("\nType the value for the exponential constant h_exp (default 200): ");
		else if(mtr=='2' || mtr=='4')
			printf("\nType the value for the exponential constant h_exp (default 61): ");
		else if(mtr=='3')
			printf("\nType the value for the exponential constant h_exp (default 400): ");
		else
			printf("\nType the value for the exponential constant h_exp (default 450): ");
		gets(string);
		h_exp = atof(string);
		if(strlen(string)<1)
		{
			if(mtr=='1')
				h_exp = 200;
			else if(mtr=='2' || mtr=='4')
				h_exp = 61;
			else if(mtr=='3')
				h_exp = 400;
			else
				h_exp = 450;
		}
		if(h_exp < 0 || h_exp > 10000)
		{
			printf("\n 0 <= h_exp <= 10000 !!\n");
			continue;
		}
		else
			break;
	}/* while(1)*/



	while(1)
	{
		printf("\n\nType the number of reflections (default 10): ");
		gets(string);
		max_ref=atoi(string);
		if(strlen(string)<1)
		{
			max_ref = 10;
			break;
		}
		if(max_ref>0 && max_ref<=10)
			break;
		else
		{
			printf("Number of reflections must be larger than 0 and less than or equal to 10!!\n");
			continue;
		}
	}




	while(1)
	{
		printf("\n\nType the angle interval in degrees (default is 5): ");
		gets(string);
		ins_ang = atof(string);
		if(strlen(string)<1)
		{
			ins_ang = 5;
			break;
		}
		else if(ins_ang>0 && ins_ang<=50)
			break;
		else
		{
			printf("Max 50 degrees!");
			continue;
		}
	}



/* Unload */
	while(1)
	{
		printf("\n\nNumber of half cycles, 0 means no unloading:\n");
		printf("\nType the number of half cycles (default 0): ");
		gets(string);
		n_cyc = atof(string);
		if(strlen(string)<1)
			n_cyc = 0;
		if(n_cyc > 1 && n_cyc <= 10)
		{
			max_cyc=vector(1,n_cyc);
			for(i=1;i<=n_cyc-1;i++)
			{
				while(1)
				{
					if(max_type=='1')
						printf("\nType the strain value for cycle no. %d (default %.4f): ",i,max_strain*i*100/n_cyc);
					else if(max_type=='2')
						printf("\nType the stress value for cycle no. %d (default %.1f): ",i,max_strain*i/n_cyc);
					else
						printf("\nType the plastic strain value for cycle no. %d (default %.4f): ",i,max_strain*i*100/n_cyc);
					gets(string);
					*(max_cyc+i) = atof(string)/100;
					if(max_type=='2')
						*(max_cyc+i) *= 100;
					if(strlen(string)<1)
						*(max_cyc+i) = max_strain*i/n_cyc;
					if(*(max_cyc+i) <= 0 || *(max_cyc+i) >= max_strain)
					{
						if(max_type=='2')
							printf("\n 0 < max_cyc < %.1f !!\n",max_strain);
						else
							printf("\n 0 < max_cyc < %.4f !!\n",max_strain*100);
						continue;
					}
					else if(i>1 && *(max_cyc+i) < *(max_cyc+i-1))
					{
						printf("\n max_cyc[i] > max_cyc[i-1]!!");
						continue;
					}
					else
						break;
				}/* while(1)*/
			}
			break;
		}
		else if(n_cyc==0)
		{
			printf("\nThere will be no unloads!");
			break;
		}
		else if(n_cyc==1)
		{
			printf("\nThere will be one unload!");
			break;
		}
		else
		{
			printf("\nMax 10 half cycles!!\n");
			continue;
		}
	}/*while(1)*/

	printf("\n\n\nList of the chosen settings:\n");
	printf("\n\nThe grain file is: %s",filename);
	printf("\nThe number of grains is: %4d",max_grain);
	printf("\nThe strain increment is: %.3f %%",ep_d*100);
	if(max_type=='1')
		printf("\nThe maximum strain is: %.3f %%",max_strain*100);
	else if(max_type=='2')
		printf("\nThe maximum load is: %.1f MPa",max_strain);
	else
		printf("\nThe maximum plastic strain is: %.3f %%",max_strain*100);
	if(mtr=='1')
		printf("\nThe material is: ISO (isotropic)\n");
	else if(mtr=='2')
		printf("\nThe material is: aluminium\n");
	else if(mtr=='3')
		printf("\nThe material is: copper\n");
	else if(mtr=='4')
		printf("\nThe material is: hybrid\n");
	else
		printf("\nThe material is: stainless steel\n");
	printf("\nThe angle interval is %.3f deg.",ins_ang);
	printf("\nThe chosen tau0 is: %.3f MPa\n",tau0);
	printf("\nThe hardening coefficients are:");
	printf("\nq = %.3f\nh_final = %.1f\nh_ratio = %.1f\nh_exp = %.1f",q_ratio,h_final,h_ratio,h_exp);
	printf("\nThere will be used %d reflections",max_ref);
	if(n_cyc>1)
	{
		printf("\n\nThere will be used %d intermediary unloads!",n_cyc-1);
		for(i=1;i<=n_cyc-1;i++)
		{
			if(max_type=='2')
				printf("\nUnload no. %d at: %.1f MPa",i,*(max_cyc+i));
			else
				printf("\nUnload no. %d at: %.4f %%",i,*(max_cyc+i)*100);
		}
	}
	else if(n_cyc==1)
		printf("\n\nThere will be one unload!");
	else
		printf("\n\nThere will be no unloads!");

	printf("\n\nOK to start calculations y/n? (default y) ");
	gets(string);
	if(*string=='n' || *string=='N')
	{
		printf("\n\nNo input file made !!");
		getc(stdin);
		exit(1);
	}
	else
	{
		if((out=fopen("sc_input.dat","w")) == NULL)
		{
			printf("\nCannot open output file!");
			ch=getc(stdin);
				exit(1);
		}
		fprintf(out," %s\n",filename);
		fprintf(out," %d\n",max_grain);
		fprintf(out," %e\n",ep_d);
		fprintf(out," %d\n %e\n",max_type,max_strain);
		fprintf(out," %d\n",mtr);
		fprintf(out," %e\n",tau0);
		fprintf(out," %e\n",q_ratio);
		fprintf(out," %e\n",h_final);
		fprintf(out," %e\n %e\n",h_ratio,h_exp);
		fprintf(out," %d\n",max_ref);
		fprintf(out," %e\n",ins_ang);
		fprintf(out," %d\n",n_cyc);
		if(n_cyc>1)
			for(i=1;i<=n_cyc-1;i++)
				fprintf(out," %e\n",*(max_cyc+i));
		fclose(out);
	}

/*
	if((ini=fopen("sc_input.dat","r")) == NULL)
	{
		printf("\nCannot open input file!");
		ch=getc(stdin);
		exit(1);
	}
	fscanf(ini,"%s",filename);
	fscanf(ini,"%d",&max_grain);
	fscanf(ini,"%lf",&ep_d);
	fscanf(ini,"%d",&max_type);
	fscanf(ini,"%lf\n",&max_strain);
	fscanf(ini,"%d\n",&mtr);
	fscanf(ini,"%lf\n",&tau0);
	fscanf(ini,"%lf\n",&q_ratio);
	fscanf(ini,"%lf\n",&h_final);
	fscanf(ini,"%lf\n",&h_ratio);
	fscanf(ini,"%lf\n",&h_exp);
	fscanf(ini,"%d",&max_ref);
	fscanf(ini,"%lf\n",&ins_ang);
	fscanf(ini,"%d\n",&n_cyc);
	if(n_cyc>1)
		for(i=1;i<=n_cyc-1;i++)
			fscanf(ini,"%f\n",&max_cyc[i]);
	fclose(ini);
	printf("grain_file: %s\n",filename);
	printf("max_grain:  %d\n",max_grain);
	printf("ep_d:       %f\n",ep_d);
	printf("max_type:   %c : %d\n",max_type,max_type);
	printf("max_strain: %f\n",max_strain);
	printf("mtr:        %c : %d\n",mtr,mtr);
	printf("tau0:       %f\n",tau0);
	printf("q_ratio:    %f\n",q_ratio);
	printf("h_final:    %f\n",h_final);
	printf("h_ratio:    %f\n",h_ratio);
	printf("h_exp:      %f\n",h_exp);
	printf("max_ref:    %d\n",max_ref);
	printf("ins_ang:    %f\n",ins_ang);
	printf("n_cyc:      %d\n",n_cyc);
	for(i=1;i<=n_cyc-1;i++)
		printf("max_cyc_%d:   %f\n",i,*(max_cyc+i));
	ch=getc(stdin);
	exit(1);
*/

	return 1;
}/*int main(void)*/
